#!/usr/bin/python
# -*- coding: utf-8 -*-
""" boto3操作swift对象存储 """


import logging
from boto3 import Session

# FORMAT = '%(asctime)-15s -8s %(message)s'
# logging.basicConfig(level=logging.DEBUG,
#                     format=FORMAT,)
log = logging.getLogger(__name__)

class S3Api(object):
    """ 操作对象存储 """

    def __init__(self,access_key,secret_key,endpoint_url,region_name='US'):
        self.access_key = access_key
        self.secret_key = secret_key
        self.endpoint_url = endpoint_url
        self.session = Session(self.access_key,self.secret_key)
        self.s3_client = self.session.client('s3', endpoint_url=endpoint_url)
        self.resource = self.session.resource('s3', endpoint_url=endpoint_url)

    def get_bucket_list(self):
        """ 获取ak sk 对应的bucket """
        return self.s3_client.list_buckets()['Buckets']

    def get_list_bucket_file(self,bucket_name,prefix=None):
        if not prefix:
            res = self.s3_client.list_objects(Bucket=bucket_name,Delimiter='/')
        else:
            if prefix:
                if not prefix.endswith('/'):
                    prefix += '/'
            res = self.s3_client.list_objects(Bucket=bucket_name, Delimiter='/',Prefix=prefix)
        dir = res.get('CommonPrefixes')
        files = res.get('Contents')
        resuts = []
        if dir:
            for o in dir:
                resuts.append(o.get('Prefix'))
        if files:
            for f in files:
                resuts.append(f.get('Key'))
        return resuts


    def add_bucket(self,bucket_name):
        try:
            res = self.s3_client.create_bucket(Bucket=bucket_name)
            if res.has_key('Location'):
                return True
        except Exception as e:
            log.exception(e)
        return False


    def del_bucket(self,bucket_name):
        """ 删除bucket  """
        try:
            res = self.s3_client.delete_bucket(Bucket=bucket_name)
            print res
            return True
        except Exception as e:
            print e
            log.exception(e)
            return False

    def download_file_link(self,bucket_name,path):
        try:
            self.s3_client.head_object(Bucket=bucket_name, Key=path)
            private_url = self.s3_client.generate_presigned_url(
                ClientMethod='get_object',
                Params={'Bucket':bucket_name,'Key':path},
                ExpiresIn=3600,HttpMethod='GET')
        except Exception as e:
            log.exception(e)
            return False,None
        return True,private_url

    def read_file(self,bucket_name,path):
        resp=self.s3_client.get_object(Bucket=bucket_name, Key=path)
        with open("/root/lxz/mysql.bak",'a') as f:
            f.write(resp['Body'].read())




    def add_file(self,bucket_name,filepath,num,acl="public-read-write",):
        """ 上传小文件 """

        for i in range(num):
            with open(filepath,'rb') as f:
                data = self.s3_client.put_object(Bucket=bucket_name,Key="data-backup/mysql_store/yangzhiwen4a273f8eda164c73859f10ef36085f2f/full-backup/%s.bak"%i,Body=f.read(),ACL=acl)
                print data

    def add_big_file(self,bucket_name,filepath):
        """ 上传你文件 """
        bucket = self.resource.Bucket(bucket_name)
        mpu = bucket.Object("data/backup/8-1.bak").initiate_multipart_upload()
        part_info = {'Parts': []}
        i = 1
        f = open(filepath,'rb')
        while 1:
            data = f.read(100*1024*1024)
            if data == b'':
                break
            part = mpu.Part(i)
            resp = part.upload(Body=data)
            part_info["Parts"].append(dict(
             PartNumber = i,
             ETag =resp['ETag']
            ))
            i += 1
            print i
        mpu.complete(MultipartUpload=part_info)
        f.close()

    def delete_file(self,bucket_name,filepath):
        """ 删除文件 """
        result = self.s3_client.delete_object(Bucket=bucket_name, Key = filepath)
        return result

    def all_file(self,bucket_name):
        response= self.s3_client.list_objects(Bucket = bucket_name,Delimiter='/')
        print "======dirs follows========"
        for o in response.get("CommonPrefixes"):
            print o.get('prefix')
        print "====Files======"
        for o in response.get('Contents'):
            print o.get("key")


if __name__=="__main__":
    access_key = ""
    secret_key = ""
    endpoint_url = ""
    s3 = S3Api(access_key,secret_key,endpoint_url)

    # print s3.get_bucket_list()
    # print s3.get_list_bucket_file("6n4ul75p")
    # print s3.add_big_file("8u4r6pwg","mysql_test.bak")
    print s3.add_big_file("8u4r6pwg","ubuntu-16.04.zip")

